@@ -67,12 +67,29 @@ placed into a Set::Object, scalars that are inserted will be flattened
 into strings, so will lose any magic (eg, tie) or other special bits
 that they went in with; only strings come out.
 
-=head1 CLASS METHODS
+=head1 CONSTRUCTORS
 
-=head2 new( [I<list>] )
+=head2 Set::Object->new( [I<list>] )
 
 Return a new C<Set::Object> containing the elements passed in I<list>.
 
+=head2 C<set(@members)>
+
+Return a new C<Set::Object> filled with C<@members>.  You have to
+explicitly import this method.
+
+B<New in Set::Object 1.22>: this function is now called as a method
+to return new sets the various methods that return a new set, such as
+C<-E<gt>intersection>, C<-E<gt>union>, etc and their overloaded
+counterparts.  The default method always returns C<Set::Object>
+objects, preserving previous behaviour and not second guessing the
+nature of your derived L<Set::Object> class.
+
+=head2 C<weak_set()>
+
+Return a new C<Set::Object::Weak>, filled with C<@members>.  You have
+to explicitly import this method.
+
 =head1 INSTANCE METHODS
 
 =head2 insert( [I<list>] )
@@ -82,7 +99,8 @@ Add items to the C<Set::Object>.
 Adding the same object several times is not an error, but any
 C<Set::Object> will contain at most one occurence of the same object.
 
-Returns the number of elements that were actually added.
+Returns the number of elements that were actually added.  As of
+Set::Object 1.23, C<undef> will not insert.
 
 =head2 includes( [I<list>] )
 
@@ -94,6 +112,10 @@ Return C<true> if B<all> the objects in I<list> are members of the
 C<Set::Object>.  I<list> may be empty, in which case C<true> is
 always returned.
 
+As of Set::Object 1.23, C<undef> will never appear to be present in
+any set (even if the set contains the empty string).  Prior to 1.23,
+there would have been a run-time warning.
+
 =head2 member( [I<item>] )
 
 =head2 element( [I<item>] )
@@ -123,6 +145,42 @@ from the C<Set::Object> is not an error.
 
 Returns the number of elements that were actually removed.
 
+As of Set::Object 1.23, removing C<undef> is safe (but having an
+C<undef> in the passed in list does not increase the return value,
+because it could never be in the set)
+
+=head2 weaken
+
+Makes all the references in the set "weak" - that is, they do not
+increase the reference count of the object they point to, just like
+L<Scalar::Util|Scalar::Util>'s C<weaken> function.
+
+This was introduced with Set::Object 1.16, and uses a brand new type
+of magic.  B<Use with caution>.  If you get segfaults when you use
+C<weaken>, please reduce your problem to a test script before
+submission.
+
+B<New:> as of Set::Object 1.19, you may use the C<weak_set> function
+to make weak sets, or C<Set::Object::Weak-E<gt>new>, or import the
+C<set> constructor from C<Set::Object::Weak> instead.  See
+L<Set::Object::Weak> for more.
+
+B<Note to people sub-classing C<Set::Object>:> this method re-blesses
+the invocant to C<Set::Object::Weak>.  Override the method C<weak_pkg>
+in your sub-class to control this behaviour.
+
+=head2 is_weak
+
+Returns a true value if this set is a weak set.
+
+=head2 strengthen
+
+Turns a weak set back into a normal one.
+
+B<Note to people sub-classing C<Set::Object>:> this method re-blesses
+the invocant to C<Set::Object>.  Override the method C<strong_pkg> in
+your sub-class to control this behaviour.
+
 =head2 invert( [I<list>] )
 
 For each item in I<list>, it either removes it or adds it to the set,
@@ -141,6 +199,20 @@ Empty this C<Set::Object>.
 Return a textual Smalltalk-ish representation of the C<Set::Object>.
 Also available as overloaded operator "".
 
+=head2 equal( I<set> )
+
+Returns a true value if I<set> contains exactly the same members as
+the invocant.
+
+Also available as overloaded operator C<==> (or C<eq>).
+
+=head2 not_equal( I<set> )
+
+Returns a false value if I<set> contains exactly the same members as
+the invocant.
+
+Also available as overloaded operator C<!=> (or C<ne>).
+
 =head2 intersection( [I<list>] )
 
 Return a new C<Set::Object> containing the intersection of the
@@ -193,6 +265,11 @@ Also available as operator C<E<gt>=>.
 Return C<true> if this C<Set::Object> is a proper superset of I<set>
 Also available as operator C<E<gt>>.
 
+=head2 is_null( I<set> )
+
+Returns a true value if this set does not contain any members, that
+is, if its size is zero.
+
 =head1 Set::Scalar compatibility methods
 
 By and large, L<Set::Object> is not and probably never will be
@@ -220,7 +297,6 @@ class method.  If you want anything fancier than this, you should
 sub-class Set::Object.
 
 
-
 =head1 FUNCTIONS
 
 The following functions are defined by the Set::Object XS code for
@@ -283,8 +359,65 @@ be a more general multi-purpose collection, then this (and C<ish_int>)
 might be a good function to use to distinguish different types of
 indexes from values.
 
+=item B<get_magic>
+
+Pass to a scalar, and get the magick wand (C<mg_obj>) used by the weak
+set implementation.  The return will be a list of integers which are
+pointers to the actual C<ISET> structure.  Whatever you do don't
+change the array :).  This is used only by the test suite, and if you
+find it useful for something then you should probably conjure up a
+test suite and send it to me, otherwise it could get pulled.
+
 =back
 
+=head1 CLASS METHODS
+
+These class methods are probably only interesting to those
+sub-classing C<Set::Object>.
+
+=over
+
+=item strong_pkg
+
+When a set that was already weak is strengthened using
+C<-E<gt>strengthen>, it gets re-blessed into this package.
+
+=item weak_pkg
+
+When a set that was NOT already weak is weakened using
+C<-E<gt>weaken>, it gets re-blessed into this package.
+
+=item tie_array_pkg
+
+When the object is accessed as an array, tie the array into this
+package.
+
+=item tie_hash_pkg
+
+When the object is accessed as a hash, tie the hash into this package.
+
+=back
+
+=head1 SERIALIZATION
+
+It is possible to serialize C<Set::Object> objects via L<Storable> and
+duplicate via C<dclone>; such support was added in release 1.04.  As
+of C<Set::Object> version 1.15, it is possible to freeze scalar items,
+too.
+
+However, the support for freezing scalar items introduced a backwards
+incompatibility.  Earlier versions than 1.15 will C<thaw> sets frozen
+using Set::Object 1.15 and later as a set with one item - an array
+that contains the actual members.
+
+Additionally, version 1.15 had a bug that meant that it would not
+detect C<freeze> protocol upgrades, instead reverting to pre-1.15
+behaviour.
+
+C<Set::Object> 1.16 and above are capable of dealing correctly with
+all serialized forms, as well as correctly aborting if a "newer"
+C<freeze> protocol is detected during C<thaw>.
+
 =head1 PERFORMANCE
 
 The following benchmark compares C<Set::Object> with using a hash to
@@ -328,12 +461,19 @@ On my computer the results are:
      H lookup:  7 secs ( 7.14 usr  0.00 sys =  7.14 cpu)
      S lookup:  6 secs ( 5.94 usr  0.00 sys =  5.94 cpu)
 
+=head1 THREAD SAFETY
+
+This module has none.
+
 =head1 AUTHOR
 
 Original Set::Object module by Jean-Louis Leroy, <jll@skynet.be>
 
-Set::Scalar compatibility, XS debugging and other maintainership
-courtesy of Sam Vilain, <samv@cpan.org>
+Set::Scalar compatibility, XS debugging, weak references support and
+general maintainership courtesy of Sam Vilain, <samv@cpan.org>.
+Maximum respect to those who send me test scripts, enhancements, etc
+as patches against my git tree, browsable at
+L<http://utsl.gen.nz/gitweb/?p=Set-Object>.
 
 =head1 LICENCE
 
@@ -343,6 +483,9 @@ and/or modified under the terms of the Perl Artistic License
 
 Portions Copyright (c) 2003 - 2005, Sam Vilain.  Same license.
 
+Portions Copyright (c) 2006, 2007, Catalyst IT (NZ) Limited.  Same
+license.
+
 =head1 SEE ALSO
 
 perl(1), perltie(1), L<Set::Scalar>, overload.pm
@@ -364,10 +507,9 @@ require AutoLoader;
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
 
-@EXPORT = qw(set);
 @EXPORT_OK = qw( ish_int is_int is_string is_double blessed reftype
-		 refaddr is_overloaded is_object is_key );
-$VERSION = '1.10';
+		 refaddr is_overloaded is_object is_key set weak_set );
+$VERSION = '1.25';
 
 bootstrap Set::Object $VERSION;
 
@@ -382,7 +524,7 @@ sub as_string
     croak "Tried to use as_string on something other than a Set::Object"
 	unless (UNIVERSAL::isa($self, __PACKAGE__));
 
-   'Set::Object(' . (join ' ', sort { $a cmp $b }
+    ref($self).'(' . (join ' ', sort { $a cmp $b }
 		     $self->members) . ')'
 }
 
@@ -401,7 +543,7 @@ sub not_equal
 
 sub union
 {
-    Set::Object->new
+    $_[0]->set
 	    ( map { $_->members() }
 	      grep { UNIVERSAL::isa($_, __PACKAGE__) }
 	      @_ );
@@ -414,7 +556,7 @@ sub op_union
     if (ref $_[0]) {
 	$other = shift;
     } else {
-	$other = __PACKAGE__->new(shift);
+	$other = $self->set(shift);
     }
 
     croak("Tried to form union between Set::Object & "
@@ -428,14 +570,12 @@ sub op_union
 sub intersection
 {
    my $s = shift;
-   return Set::Object->new() unless $s;
-
-   my $rem = __PACKAGE__->new($s->members);
+   my $rem = $s->set($s->members);
 
    while ($s = shift)
    {
        if (!ref $s) {
-	   $s = __PACKAGE__->new($s);
+	   $s = $rem->new($s);
        }
 
        croak("Tried to form intersection between Set::Object & "
@@ -454,7 +594,7 @@ sub op_intersection
     if (ref $_[0]) {
 	$s2 = shift;
     } else {
-	$s2 = __PACKAGE__->new(shift);
+	$s2 = $s1->set(shift);
     }
     my $r = shift;
     if ( $r ) {
@@ -481,9 +621,9 @@ sub difference
 
    my $s;
    if ( $r ) {
-       $s = Set::Object->new( grep { !$s1->includes($_) } $s2->members );
+       $s = $s2->set( grep { !$s1->includes($_) } $s2->members );
    } else {
-       $s = Set::Object->new( grep { !$s2->includes($_) } $s1->members );
+       $s = $s1->set( grep { !$s2->includes($_) } $s1->members );
    }
    $s;
 }
@@ -502,7 +642,7 @@ sub op_invert
 	  ."`$other'")
 	if ref $other and not UNIVERSAL::isa($other, __PACKAGE__);
 
-    my $result = Set::Object->new( $self->members() );
+    my $result = $self->set( $self->members() );
     $result->invert( $other->members() );
     return $result;
 
@@ -809,14 +949,32 @@ sub is_key {
 sub STORABLE_freeze {
     my $obj = shift;
     my $am_cloning = shift;
-    return ("", $obj->members);
+    return ("v3-" . ($obj->is_weak ? "w" : "s"), [ $obj->members ]);
 }
 
-use Devel::Peek qw(Dump);
+#use Devel::Peek qw(Dump);
 
 sub STORABLE_thaw {
     #print Dump $_ foreach (@_);
 
+    $DB::single = 1;
+    if ( $_[2] ) {
+	if ( $_[2] eq "v2" ) {
+	    @_ = (@_[0,1], "", @{ $_[3] });
+	}
+	elsif ( $_[2] =~ m/^v3-(w|s)/ ) {
+	    @_ = (@_[0,1], "", @{ $_[3] });
+	    if ( $1 eq "w" ) {
+		my $self = shift;
+		$self->_STORABLE_thaw(@_);
+		$self->weaken();
+		return;
+	    }
+	} else {
+	    croak("Unrecognised Set::Object Storable version $_[2]");
+	}
+    }
+
     goto &_STORABLE_thaw;
     #print "Got here\n";
 }
@@ -869,7 +1027,12 @@ sub compare {
 	    }
 	}
     } else {
-	return "disjoint";
+	if ($self->size || $other->size) {
+	    return "disjoint";
+	} else {
+	    # both sets are empty
+	    return "equal";
+	}
     }
 }
 
@@ -881,7 +1044,7 @@ sub is_disjoint {
     return !($self*$other)->size;
 }
 
-use Data::Dumper;
+#use Data::Dumper;
 sub as_string_callback {
     shift;
     if ( @_ ) {
@@ -911,8 +1074,37 @@ sub member {
 }
 
 sub set {
+    if (eval { $_[0]->isa(__PACKAGE__) }) {
+    	shift;
+    }
     __PACKAGE__->new(@_);
 }
+sub weak_set {
+    my $self = __PACKAGE__->new();
+    $self->weaken;
+    $self->insert(@_);
+    return $self;
+}
+
+require Set::Object::Weak;
+sub weaken {
+    my $self = shift;
+    $self->_weaken;
+    bless $self, $self->weak_pkg;
+}
+
+sub strengthen {
+    my $self = shift;
+    $self->_strengthen;
+    bless $self, $self->strong_pkg;
+}
+
+sub weak_pkg {
+    "Set::Object::Weak";
+}
+sub strong_pkg {
+    "Set::Object";
+}
 1;
 
 __END__
